@@ -1,18 +1,18 @@
 #include "Common/Options.h"
 #include "DataLayer/Options.h"
-#include "ContigGraph.h"
 #include "ContigNode.h"
 #include "ContigProperties.h"
-#include "DirectedGraph.h"
 #include "FastaReader.h"
-#include "GraphIO.h"
-#include "GraphUtil.h"
-#include "HashMap.h"
 #include "Iterator.h"
 #include "Kmer.h"
 #include "StringUtil.h"
 #include "SuffixArray.h"
 #include "Uncompress.h"
+#include "UnorderedMap.h"
+#include "Graph/ContigGraph.h"
+#include "Graph/DirectedGraph.h"
+#include "Graph/GraphIO.h"
+#include "Graph/GraphUtil.h"
 #include <cassert>
 #include <cctype>
 #include <cstdlib>
@@ -138,7 +138,7 @@ static void addOverlapsSA(Graph& g, const vector<Kmer>& prefixes)
 }
 
 /** An index of suffixes of k-1 bp. */
-typedef hash_map<Kmer, vector<ContigNode>, hashKmer> SuffixMap;
+typedef unordered_map<Kmer, vector<ContigNode>, hashKmer> SuffixMap;
 
 /** Read contigs. Add contig properties to the graph. Add prefixes to
  * the collection and add suffixes to their index.
@@ -172,9 +172,9 @@ static void readContigs(const string& path,
 		prefixes.push_back(reverseComplement(suffix));
 
 		// Add the suffix to the index.
-		ContigID::insert(rec.id);
 		ContigProperties vp(seq.length(), getCoverage(rec.comment));
 		ContigNode u = add_vertex(vp, g);
+		put(vertex_name, g, u, rec.id);
 		suffixMap[suffix].push_back(u);
 		suffixMap[reverseComplement(prefix)].push_back(~u);
 	}
