@@ -7,7 +7,9 @@
 #include "IOUtil.h"
 #include "Log.h"
 #include "SequenceCollection.h"
+#include "StringUtil.h"
 #include "Timer.h"
+#include <algorithm>
 #include <cctype>
 #include <climits> // for UINT_MAX
 #include <cmath>
@@ -39,6 +41,32 @@ void generateSequencesFromExtension(const Kmer& currSeq,
 	}
 }
 
+/** Load k-mer with coverage data.
+ * @return the number of k-mer loaded
+ */
+static size_t loadKmer(ISequenceCollection& g, FastaReader& in)
+{
+	assert(opt::rank == -1);
+	size_t count = 0;
+	for (FastaRecord rec; in >> rec;) {
+		assert(rec.seq.size() == opt::kmerSize);
+		istringstream iss(rec.id);
+		float coverage = 1;
+		iss >> coverage;
+		assert(iss);
+		assert(iss.eof());
+		g.add(Kmer(rec.seq), max(1, (int)ceilf(coverage)));
+
+		if (++count % 1000000 == 0) {
+			logger(1) << "Read " << count << " k-mer. ";
+			g.printLoad();
+		}
+		g.pumpNetwork();
+	}
+	assert(in.eof());
+	return count;
+}
+
 /** Load sequence data into the collection. */
 void loadSequences(ISequenceCollection* seqCollection, string inFile)
 {
@@ -56,6 +84,11 @@ void loadSequences(ISequenceCollection* seqCollection, string inFile)
 	size_t count = 0, count_good = 0,
 			 count_small = 0, count_nonACGT = 0;
 	FastaReader reader(inFile.c_str(), FastaReader::FOLD_CASE);
+	if (endsWith(inFile, ".jf") || endsWith(inFile, ".jfq")) {
+		// Load k-mer with coverage data.
+		count = loadKmer(*seqCollection, reader);
+		count_good = count;
+	} else
 	for (Sequence seq; reader >> seq;) {
 		size_t len = seq.length();
 		if (opt::kmerSize > len) {
