@@ -129,7 +129,7 @@ next_record:
 	unsigned qualityOffset = 0;
 	if (recordType == EOF || m_in.tellg() >= m_end) {
 		m_in.seekg(0, ios::end);
-		m_in.get();
+		m_in.clear(std::ios::eofbit | std::ios::failbit);
 		return s;
 	} else if (recordType == '>' || recordType == '@') {
 		// Read the header.
@@ -185,7 +185,6 @@ next_record:
 			}
 			ignoreLines(1);
 			getline(q);
-			checkSeqQual(s, q);
 		} else
 			q.clear();
 
@@ -202,7 +201,7 @@ next_record:
 			assert(s.length() > 2);
 			anchor = colourToNucleotideSpace(s[0], s[1]);
 			s.erase(0, 2);
-			q.erase(0, 2);
+			q.erase(0, 1);
 		} else if (opt::trimMasked) {
 			// Removed masked (lower case) sequence at the beginning
 			// and end of the read.
@@ -215,6 +214,8 @@ next_record:
 				q.erase(0, trimFront);
 			}
 		}
+		if (!q.empty())
+			checkSeqQual(s, q);
 		if (flagFoldCase())
 			transform(s.begin(), s.end(), s.begin(), ::toupper);
 
@@ -323,9 +324,15 @@ next_record:
 
 		size_t trimFront = q.find_first_of(goodQual);
 		size_t trimBack = q.find_last_of(goodQual) + 1;
-		if (trimFront > 0 || trimBack < q.length()) {
+		if (trimFront >= trimBack) {
+			// The entire read is poor quality.
+			s.erase(1);
+			q.erase(1);
+		} else if (trimFront > 0 || trimBack < q.length()) {
 			s.erase(trimBack);
 			s.erase(0, trimFront);
+			q.erase(trimBack);
+			q.erase(0, trimFront);
 		}
 	}
 
