@@ -1,17 +1,18 @@
 #include "config.h"
 #include "Common/Options.h"
-#include "ContigGraph.h"
-#include "ContigGraphAlgorithms.h"
 #include "ContigID.h"
 #include "ContigLength.h"
 #include "ContigPath.h"
-#include "DirectedGraph.h"
-#include "DotIO.h"
-#include "GraphAlgorithms.h"
-#include "GraphUtil.h"
+#include "Functional.h" // for mem_var
 #include "IOUtil.h"
 #include "Iterator.h"
 #include "Uncompress.h"
+#include "Graph/Assemble.h"
+#include "Graph/ContigGraph.h"
+#include "Graph/DirectedGraph.h"
+#include "Graph/DotIO.h"
+#include "Graph/GraphAlgorithms.h"
+#include "Graph/GraphUtil.h"
 #include <boost/tuple/tuple.hpp>
 #include <algorithm>
 #include <cassert>
@@ -53,8 +54,8 @@ static const char USAGE_MESSAGE[] =
 "  -k, --kmer=KMER_SIZE  k-mer size\n"
 "  -s, --seed-length=L   minimum length of a seed contig [0]\n"
 "  -o, --out=FILE        write result to FILE\n"
-"      --greedy          use the greedy algorithm [default]\n"
-"      --no-greedy       do not use the greedy algorithm\n"
+"      --no-greedy       use the non-greedy algorithm [default]\n"
+"      --greedy          use the greedy algorithm\n"
 "  -g, --graph=FILE      write the path overlap graph to FILE\n"
 "  -j, --threads=N       use N parallel threads [1]\n"
 "  -v, --verbose         display verbose output\n"
@@ -72,7 +73,7 @@ namespace opt {
 	static unsigned seedLen;
 
 	/** Use a greedy algorithm. */
-	static int greedy = true;
+	static int greedy;
 
 	/** Write the path overlap graph to this file. */
 	static string graphPath;
@@ -233,7 +234,7 @@ static ContigPath getPath(const ContigPathMap& paths, ContigNode u)
 	assert(it != paths.end());
 	ContigPath path = it->second;
 	if (u.sense())
-		path.reverseComplement();
+		reverseComplement(path.begin(), path.end());
 	return path;
 }
 
@@ -256,7 +257,7 @@ static void findPathOverlaps(const ContigPathMap& paths,
 
 		ContigPath path2 = path2It->second;
 		if (seed2.sense())
-			path2.reverseComplement();
+			reverseComplement(path2.begin(), path2.end());
 		addOverlapEdge(gout, seed2, seed1, path1, seed2, path2);
 	}
 }
@@ -279,7 +280,7 @@ static unsigned mergePaths(ContigPath& path,
 
 		ContigPath path2 = path2It->second;
 		if (pivot.sense())
-			path2.reverseComplement();
+			reverseComplement(path2.begin(), path2.end());
 		ContigPath consensus = align(path, path2, pivot);
 		if (consensus.empty()) {
 			invalid.push_back(pivot);
@@ -311,7 +312,7 @@ static ContigPath mergePath(
 	assert(path1It != paths.end());
 	ContigPath path(path1It->second);
 	if (seedPath.front().sense())
-		path.reverseComplement();
+		reverseComplement(path.begin(), path.end());
 	if (opt::verbose > 1)
 #pragma omp critical(cout)
 		cout << "\n* " << seedPath << '\n'
@@ -324,7 +325,7 @@ static ContigPath mergePath(
 		assert(path2It != paths.end());
 		ContigPath path2 = path2It->second;
 		if (seed2.sense())
-			path2.reverseComplement();
+			reverseComplement(path2.begin(), path2.end());
 
 		ContigNode pivot
 			= find(path.begin(), path.end(), seed2) != path.end()
@@ -455,7 +456,7 @@ static ContigID identifySubsumedPaths(
 			continue;
 		ContigPath path2 = path2It->second;
 		if (pivot.sense())
-			path2.reverseComplement();
+			reverseComplement(path2.begin(), path2.end());
 		ContigPath consensus = align(path, path2, pivot);
 		if (consensus.empty())
 			continue;
@@ -672,7 +673,7 @@ static void assemblePathGraph(
 		PathGraph& pathGraph, ContigPathMap& paths)
 {
 	ContigPaths seedPaths;
-	assemble(pathGraph, back_inserter(seedPaths));
+	assembleDFS(pathGraph, back_inserter(seedPaths));
 	ContigPaths mergedPaths = mergeSeedPaths(paths, seedPaths);
 	if (opt::verbose > 1)
 		cout << '\n';
@@ -682,7 +683,7 @@ static void assemblePathGraph(
 			it1 != seedPaths.end(); ++it1) {
 		const ContigPath& path(mergedPaths[it1 - seedPaths.begin()]);
 		ContigPath pathrc(path);
-		pathrc.reverseComplement();
+		reverseComplement(pathrc.begin(), pathrc.end());
 		for (ContigPath::const_iterator it2 = it1->begin();
 				it2 != it1->end(); ++it2) {
 			ContigNode seed(*it2);
@@ -707,6 +708,8 @@ static void assemblePathGraph(
 /** Read a set of paths from the specified file. */
 static ContigPathMap readPaths(const string& filePath)
 {
+	if (opt::verbose > 0)
+		cerr << "Reading `" << filePath << "'..." << endl;
 	ifstream in(filePath.c_str());
 	assert_good(in, filePath);
 
@@ -830,6 +833,8 @@ int main(int argc, char** argv)
 		omp_set_num_threads(opt::threads);
 #endif
 
+	if (opt::verbose > 0)
+		cerr << "Reading `" << argv[optind] << "'..." << endl;
 	g_contigLengths = readContigLengths(argv[optind++]);
 	ContigPathMap originalPathMap = readPaths(argv[optind++]);
 
@@ -1252,11 +1257,26 @@ static ContigPath align(
 		const ContigPath& path1, const ContigPath& path2,
 		ContigNode pivot, dir_type& orientation)
 {
-	if (path1 == path2 && &path1 != &path2) {
-		// These two paths are identical. Ignore the trivial alignment
-		// when aligning a path to itself.
+	if (&path1 == &path2) {
+		// Ignore the trivial alignment when aligning a path to
+		// itself.
+	} else if (path1 == path2) {
+		// These two paths are identical.
 		orientation = DIR_B;
 		return path1;
+	} else {
+		ContigPath::const_iterator it
+			= search(path1.begin(), path1.end(),
+				path2.begin(), path2.end());
+		if (it != path1.end()) {
+			// path2 is subsumed in path1.
+			// Determine the orientation of the edge.
+			orientation
+				= it == path1.begin() ? DIR_R
+				: it + path2.size() == path1.end() ? DIR_F
+				: DIR_B;
+			return path1;
+		}
 	}
 
 	// Find a suitable pivot.
