@@ -5,16 +5,15 @@
 
 #include "config.h"
 #include "Common/Options.h"
-#include "ContigGraph.h"
-#include "ContigGraphAlgorithms.h"
 #include "ContigProperties.h"
-#include "DirectedGraph.h"
 #include "Estimate.h"
 #include "FastaReader.h"
-#include "GraphIO.h"
 #include "IOUtil.h"
-#include "MapGraph.h"
 #include "Uncompress.h"
+#include "Graph/ContigGraph.h"
+#include "Graph/ContigGraphAlgorithms.h"
+#include "Graph/DirectedGraph.h"
+#include "Graph/GraphIO.h"
 #include <algorithm>
 #include <cassert>
 #include <cctype>
@@ -23,8 +22,6 @@
 #include <fstream>
 #include <getopt.h>
 #include <iostream>
-#include <map>
-#include <set>
 #include <sstream>
 #include <string>
 #include <vector>
@@ -173,10 +170,12 @@ static FastaRecord newContig(const ContigNode& t, const ContigNode& h,
 			comment.str(), seq);
 }
 
+/** An overlap of two sequences. */
 struct Overlap {
-	const Estimate est;
-	const unsigned overlap;
-	const bool mask;
+	Estimate est;
+	unsigned overlap;
+	bool mask;
+
 	Overlap() : est(), overlap(UINT_MAX), mask(false) { }
 	Overlap(const Estimate& est, unsigned overlap, bool mask)
 		: est(est), overlap(overlap), mask(mask) { }
@@ -223,27 +222,8 @@ static FastaRecord createGapContig(
 
 /** The scaffold graph. Edges join two blunt contigs that are joined
  * by a distance estimate. */
-typedef map<ContigNode, map<ContigNode, Overlap> > OverlapGraph;
-
-/** Remove the out-edges of vertex u. */
-static void clearOutEdges(ContigNode u, OverlapGraph& g)
-{
-	typedef graph_traits<OverlapGraph>::adjacency_iterator
-		adjacency_iterator;
-	pair<adjacency_iterator, adjacency_iterator>
-		adj = adjacent_vertices(u, g);
-	for (adjacency_iterator it = adj.first; it != adj.second; ++it) {
-		assert(~*it != u);
-		remove_edge(~*it, ~u, g);
-	}
-	clear_out_edges(u, g);
-}
-
-/** Remove the in-edges of vertex u. */
-static void clearInEdges(ContigNode u, OverlapGraph& g)
-{
-	clearOutEdges(~u, g);
-}
+typedef ContigGraph<DirectedGraph<
+	no_property, Overlap> > OverlapGraph;
 
 static void findOverlap(const Graph& g,
 		ContigID refID, bool rc, const Estimate& est,
@@ -256,7 +236,8 @@ static void findOverlap(const Graph& g,
 	const ContigNode& pair = est.contig;
 	const ContigNode& t = rc ? pair : ref;
 	const ContigNode& h = rc ? ref : pair;
-	if (g.out_degree(t) > 0 || g.in_degree(h) > 0)
+	if (g.out_degree(t) > 0 || g.in_degree(h) > 0
+			|| edge(t, h, out).second)
 		return;
 
 	bool mask = false;
@@ -265,11 +246,8 @@ static void findOverlap(const Graph& g,
 		? findOverlap(t, h, mask) : 0;
 	if (mask && !opt::mask)
 		return;
-	if (overlap > 0 || opt::scaffold) {
-		Overlap ep(est, overlap, mask);
-		add_edge(t, h, ep, out);
-		add_edge(~h, ~t, ep, out);
-	}
+	if (overlap > 0 || opt::scaffold)
+		add_edge(t, h, Overlap(est, overlap, mask), out);
 }
 
 static void readContigs(const char *contigPath)
@@ -358,7 +336,7 @@ int main(int argc, char** argv)
 	assert_good(in, estPath);
 
 	// Find overlapping contigs.
-	OverlapGraph scaffoldGraph;
+	OverlapGraph scaffoldGraph(graph.num_vertices() / 2);
 	for (EstimateRecord er; in >> er;) {
 		for (int rc = false; rc <= true; ++rc) {
 			const vector<Estimate>& ests = er.estimates[rc];
@@ -385,11 +363,13 @@ int main(int argc, char** argv)
 		out_edge_iterator;
 
 	/** The overlapping edges (d<0) of scaffoldGraph. */
-	OverlapGraph overlapGraph;
+	OverlapGraph overlapGraph(graph.num_vertices() / 2);
 
 	/** The canonical edges of scaffoldGraph. */
-	typedef set<edge_descriptor> Edges;
+	unsigned numOverlaps = num_edges(scaffoldGraph) / 2;
+	typedef vector<edge_descriptor> Edges;
 	Edges edges;
+	edges.reserve(numOverlaps);
 
 	// Create the set of canonical edges and the overlap subgraph.
 	std::pair<vertex_iterator, vertex_iterator>
@@ -399,22 +379,23 @@ int main(int argc, char** argv)
 			vit = out_edges(*u, scaffoldGraph);
 		for (out_edge_iterator e = vit.first; e != vit.second; ++e) {
 			vertex_descriptor v = target(*e, scaffoldGraph);
-			if (edges.count(edge_descriptor(~v, ~*u)) == 0)
-				edges.insert(*e);
+			assert(*u != v);
+			if (v < *u)
+				continue;
+			edges.push_back(*e);
 			const Overlap& ep = get(edge_bundle, scaffoldGraph, e);
-			if (ep.overlap > 0) {
+			if (ep.overlap > 0)
 				add_edge(*u, v, ep, overlapGraph);
-				add_edge(~v, ~*u, ep, overlapGraph);
-			}
 		}
 	}
+	assert(edges.size() == numOverlaps);
 
 	// First, give priority to overlapping edges (not scaffolded).
 	for (Edges::const_iterator it = edges.begin();
 			it != edges.end(); ++it) {
 		const ContigNode& t = source(*it, overlapGraph),
 			  h = target(*it, overlapGraph);
-		if (overlapGraph[t].count(h) == 0) {
+		if (!edge(t, h, overlapGraph).second) {
 			// This edge is scaffolded.
 			continue;
 		}
@@ -426,8 +407,8 @@ int main(int argc, char** argv)
 			graph.add_edge(t, h, overlap);
 
 			// Clear the out-edges of t and the in-edges of h.
-			clearOutEdges(t, scaffoldGraph);
-			clearInEdges(h, scaffoldGraph);
+			clear_out_edges(t, scaffoldGraph);
+			clear_in_edges(h, scaffoldGraph);
 		} else
 			stats.ambiguous++;
 	}
@@ -438,7 +419,7 @@ int main(int argc, char** argv)
 			it != edges.end(); ++it) {
 		const ContigNode& t = source(*it, scaffoldGraph),
 			  h = target(*it, scaffoldGraph);
-		if (scaffoldGraph[t].count(h) == 0) {
+		if (!edge(t, h, scaffoldGraph).second) {
 			// This edge involved a vertex that has already been used
 			// and removed.
 			continue;
