@@ -1162,7 +1162,7 @@ performNetworkAssembly(ISequenceCollection* seqCollection,
 			"Removed " << m_lowCoverageKmer << " k-mer in "
 				<< m_lowCoverageContigs << " low-coverage contigs.\n";
 	} else
-		cout << "Assembled " << numAssembled.second << " k-mer in "
+		logger(0) << "Assembled " << numAssembled.second << " k-mer in "
 			<< numAssembled.first << " contigs.\n";
 	return numAssembled;
 }
@@ -1308,12 +1308,15 @@ void NetworkSequenceCollection::processSequenceExtensionPop(
 }
 
 /** Add a k-mer to this collection. */
-void NetworkSequenceCollection::add(const Kmer& seq)
+void NetworkSequenceCollection::add(const Kmer& seq,
+		unsigned coverage)
 {
-	if (isLocal(seq))
-		m_data.add(seq);
-	else
+	if (isLocal(seq)) {
+		m_data.add(seq, coverage);
+	} else {
+		assert(coverage == 1);
 		m_comm.sendSeqAddMessage(computeNodeID(seq), seq);
+	}
 }
 
 /** Remove a k-mer from this collection. */
