@@ -4,8 +4,9 @@
 #include "IOUtil.h"
 #include "MemoryUtil.h"
 #include "SAM.h"
-#include "StringUtil.h" // for toSI
+#include "StringUtil.h"
 #include "Uncompress.h"
+#include "UnorderedMap.h"
 #include <algorithm>
 #include <climits>
 #include <cmath>
@@ -98,10 +99,10 @@ static ofstream fragFile;
 static Histogram histogram;
 
 /** A map of read IDs to alignments. */
-typedef hash_map<string, AlignmentVector> ReadAlignMap;
+typedef unordered_map<string, AlignmentVector> ReadAlignMap;
 
 /** A map of contig IDs to distance estimates. */
-typedef hash_map<string, EstimateRecord> EstimateMap;
+typedef unordered_map<string, EstimateRecord> EstimateMap;
 static EstimateMap estMap;
 
 static bool checkUniqueAlignments(const AlignmentVector& alignVec);
@@ -600,12 +601,6 @@ static bool checkUniqueAlignments(const AlignmentVector& alignVec)
 	return true;
 }
 
-static bool endsWith(const string& s, const string& suffix)
-{
-	ssize_t i = s.length() - suffix.length();
-	return i < 0 ? false : s.substr(i) == suffix;
-}
-
 static bool replaceSuffix(string& s,
 		const string& suffix0, const string& suffix1)
 {
