@@ -3,23 +3,27 @@
  * Written by Shaun Jackman <sjackman@bcgsc.ca>.
  */
 
+#include "dialign.h"
 #include "config.h"
 #include "Common/Options.h"
 #include "ConstString.h"
-#include "ContigGraph.h"
-#include "ContigGraphAlgorithms.h"
 #include "ContigPath.h"
 #include "ContigProperties.h"
-#include "DepthFirstSearch.h"
-#include "DirectedGraph.h"
 #include "FastaReader.h"
-#include "GraphIO.h"
-#include "GraphUtil.h"
 #include "IOUtil.h"
 #include "Iterator.h"
 #include "Sequence.h"
 #include "Uncompress.h"
 #include "alignGlobal.h"
+#include "Graph/ContigGraph.h"
+#include "Graph/ContigGraphAlgorithms.h"
+#include "Graph/DepthFirstSearch.h"
+#include "Graph/DirectedGraph.h"
+#include "Graph/GraphIO.h"
+#include "Graph/GraphUtil.h"
+#include "Graph/PopBubbles.h"
+#include <boost/lambda/bind.hpp>
+#include <boost/lambda/lambda.hpp>
 #include <algorithm>
 #include <climits> // for UINT_MAX
 #include <fstream>
@@ -38,7 +42,11 @@
 #endif
 
 using namespace std;
+using namespace boost::lambda;
 using boost::tie;
+#if !__GXX_EXPERIMENTAL_CXX0X__
+using boost::cref;
+#endif
 
 #define PROGRAM "PopBubbles"
 
@@ -55,6 +63,7 @@ static const char USAGE_MESSAGE[] =
 "  ADJ    contig adjacency graph\n"
 "\n"
 "  -k, --kmer=N          k-mer size\n"
+"  -a, --branches=N      maximum number of branches, default: 2\n"
 "  -b, --bubble-length=N pop bubbles shorter than N bp\n"
 "                        default is 10000\n"
 "  -p, --identity=REAL   minimum identity, default: 0.9\n"
@@ -75,6 +84,9 @@ static const char USAGE_MESSAGE[] =
 namespace opt {
 	unsigned k; // used by ContigProperties
 
+	/** Maximum number of branches. */
+	static unsigned maxBranches = 2;
+
 	/** Pop bubbles shorter than this threshold. */
 	static unsigned maxLength = 10000;
 
@@ -97,13 +109,17 @@ namespace opt {
 
 	/** Number of threads. */
 	static int threads = 1;
+	static int dialign_debug;
+	static string dialign_score;
+	static string dialign_prob;
 }
 
-static const char shortopts[] = "b:c:g:j:k:p:v";
+static const char shortopts[] = "a:b:c:g:j:k:p:vD:M:P:";
 
 enum { OPT_HELP = 1, OPT_VERSION };
 
 static const struct option longopts[] = {
+	{ "branches",      required_argument, NULL, 'a' },
 	{ "bubble-length", required_argument, NULL, 'b' },
 	{ "coverage",      required_argument, NULL, 'c' },
 	{ "dot",           no_argument,       &opt::dot, 1, },
@@ -116,6 +132,9 @@ static const struct option longopts[] = {
 	{ "verbose",       no_argument,       NULL, 'v' },
 	{ "help",          no_argument,       NULL, OPT_HELP },
 	{ "version",       no_argument,       NULL, OPT_VERSION },
+	{ "dialign-d",   required_argument, NULL, 'D' },
+	{ "dialign-m",   required_argument, NULL, 'M' },
+	{ "dialign-p",   required_argument, NULL, 'P' },
 	{ NULL, 0, NULL, 0 }
 };
 
@@ -127,31 +146,6 @@ typedef ContigGraph<DirectedGraph<ContigProperties, Distance> > Graph;
 typedef Graph::vertex_descriptor vertex_descriptor;
 typedef Graph::adjacency_iterator adjacency_iterator;
 
-/** Record a topological order of the vertices. */
-template <typename OutIt>
-struct TopoVisitor : public boost::default_dfs_visitor
-{
-	TopoVisitor(OutIt it) : m_it(it) { }
-
-	template <typename Vertex, typename Graph>
-	void finish_vertex(const Vertex& u, Graph&) { *m_it++ = u; }
-
-  private:
-	OutIt m_it;
-};
-
-/** Record a topological order of the vertices. */
-template <typename Graph, typename It>
-static void topologicalSort(const Graph& g, It it)
-{
-	using boost::default_color_type;
-	using boost::vector_property_map;
-	typedef vector_property_map<
-		default_color_type, ContigNodeIndexMap> ColorMap;
-	depthFirstSearch(g, TopoVisitor<It>(it),
-			ColorMap(num_vertices(g)));
-}
-
 /** Return the distance from vertex u to v. */
 static int getDistance(const Graph& g,
 		vertex_descriptor u, vertex_descriptor v)
@@ -227,24 +221,100 @@ static unsigned getLength(const Graph* g, vertex_descriptor v)
 	return (*g)[v].length;
 }
 
-/** Align the sequences of [first,last).
- * @return the identity of the global alignment
+/** Align the specified pair of sequences.
+ * @return the number of matches and size of the consensus
  */
-template <typename It>
-static float getAlignmentIdentity(It first, It last)
+static pair<unsigned, unsigned> alignPair(
+		const string& seqa, const string& seqb)
 {
-	assert(distance(first, last) == 2);
-	(void)last;
-	string seqa = getSequence(*first);
-	++first;
-	string seqb = getSequence(*first);
-
 	NWAlignment alignment;
 	unsigned matches = alignGlobal(seqa, seqb, alignment);
 	if (opt::verbose > 2)
 #pragma omp critical(cerr)
 		cerr << alignment;
-	return (float)matches / alignment.size();
+	return make_pair(matches, alignment.size());
+}
+
+/** Align the specified sequences.
+ * @return the number of matches and size of the consensus
+ */
+static pair<unsigned, unsigned> alignMulti(const vector<string>& seqs)
+{
+	string alignment;
+	unsigned matches;
+	string consensus = dialign(seqs, alignment, matches);
+	if (opt::verbose > 2)
+#pragma omp critical(cerr)
+		cerr << alignment << consensus << '\n';
+	return make_pair(matches, consensus.size());
+}
+
+/** Align the specified sequences.
+ * @return the number of matches and size of the consensus
+ */
+static pair<unsigned, unsigned> align(const vector<string>& seqs)
+{
+	assert(seqs.size() > 1);
+	if (seqs.size() == 2)
+		return alignPair(seqs[0], seqs[1]);
+	else
+		return alignMulti(seqs);
+}
+
+/** Align the sequences of [first,last).
+ * @param t the vertex to the left of the bubble
+ * @param v the vertex to the right of the bubble
+ * @return the identity of the global alignment
+ */
+template <typename It>
+static float getAlignmentIdentity(const Graph& g,
+		vertex_descriptor t, vertex_descriptor v,
+		It first, It last)
+{
+	unsigned nbranches = distance(first, last);
+	vector<int> inDists(nbranches);
+	transform(first, last, inDists.begin(),
+			bind(getDistance, cref(g), t, _1));
+	vector<int> outDists(nbranches);
+	transform(first, last, outDists.begin(),
+			bind(getDistance, cref(g), _1, v));
+	vector<int> insertLens(nbranches);
+	transform(first, last, insertLens.begin(),
+			bind(getDistance, cref(g), t, _1)
+				+ bind(getLength, &g, _1)
+				+ bind(getDistance, cref(g), _1, v));
+
+	int max_in_overlap = -(*min_element(inDists.begin(),
+			inDists.end()));
+	assert(max_in_overlap >= 0);
+	int max_out_overlap = -(*min_element(outDists.begin(),
+			outDists.end()));
+	assert(max_out_overlap >= 0);
+	int min_insert_len = *min_element(insertLens.begin(),
+			insertLens.end());
+	int max_insert_len = *max_element(insertLens.begin(),
+			insertLens.end());
+
+	float max_identity =
+		(float)(min_insert_len + max_in_overlap + max_out_overlap) /
+		(max_insert_len + max_in_overlap + max_out_overlap);
+	if (min_insert_len <= 0 || max_identity < opt::identity)
+		return max_identity;
+
+	vector<string> seqs(nbranches);
+	transform(first, last, seqs.begin(), getSequence);
+	for (unsigned i = 0; i < seqs.size(); i++) {
+		// Remove the overlapping sequence.
+		int n = seqs[i].size();
+		int l = -inDists[i], r = -outDists[i];
+		assert(n > l + r);
+		seqs[i] = seqs[i].substr(l, n - l - r);
+	}
+
+	unsigned matches, consensusSize;
+	tie(matches, consensusSize) = align(seqs);
+	return (float)(matches + max_in_overlap + max_out_overlap) /
+		(consensusSize + max_in_overlap + max_out_overlap);
 }
 
 /** Pop the specified bubble if it is a simple bubble.
@@ -295,8 +365,7 @@ static bool popSimpleBubble(Graph* pg, vertex_descriptor v)
 		cerr << "-> " << tail << '\n';
 	}
 
-	const unsigned MAX_BRANCHES = opt::identity > 0 ? 2 : UINT_MAX;
-	if (nbranches > MAX_BRANCHES) {
+	if (nbranches > opt::maxBranches) {
 		// Too many branches.
 #pragma omp atomic
 		g_count.tooMany++;
@@ -323,7 +392,7 @@ static bool popSimpleBubble(Graph* pg, vertex_descriptor v)
 	}
 
 	float identity = opt::identity == 0 ? 0
-		: getAlignmentIdentity(adj.first, adj.second);
+		: getAlignmentIdentity(g, v, tail, adj.first, adj.second);
 	bool dissimilar = identity < opt::identity;
 	if (opt::verbose > 1)
 #pragma omp critical(cerr)
@@ -342,83 +411,6 @@ static bool popSimpleBubble(Graph* pg, vertex_descriptor v)
 	return true;
 }
 
-/** Return true if the specified sequence of vertices is a bubble. */
-template <typename Graph, typename It>
-static bool isBubble(const Graph& g, It first, It last)
-{
-	typedef typename graph_traits<Graph>::adjacency_iterator Ait;
-	typedef typename graph_traits<Graph>::vertex_descriptor V;
-	assert(last - first > 1);
-	if (last - first == 2)
-		return false; // unambiguous edge
-	if (*first == ~last[-1])
-		return false; // palindrome
-	set<V> targets(first, first + 1);
-	for (It it = first; it != last - 1; ++it) {
-		pair<Ait, Ait> adj = adjacent_vertices(*it, g);
-		targets.insert(adj.first, adj.second);
-	}
-	set<V> sources(last - 1, last);
-	for (It it = first + 1; it != last; ++it) {
-		pair<Ait, Ait> adj = adjacent_vertices(~*it, g);
-		transform(adj.first, adj.second,
-				inserter(sources, sources.end()),
-				mem_fun_ref(&V::operator~));
-	}
-	set<V> bubble(first, last);
-	return sources == bubble && targets == bubble;
-}
-
-typedef vector<ContigNode> Bubble;
-typedef vector<Bubble> Bubbles;
-
-/** Discover bubbles. */
-static Bubbles discoverBubbles(const Graph& g)
-{
-	typedef graph_traits<Graph>::vertex_descriptor V;
-
-	vector<V> topo(num_vertices(g));
-	topologicalSort(g, topo.rbegin());
-
-	Bubbles bubbles;
-	typedef vector<V>::const_iterator It;
-	for (It first = topo.begin(); first != topo.end(); ++first) {
-		int sum = out_degree(*first, g);
-		if (sum < 2)
-			continue;
-		if (opt::verbose > 3)
-			cerr << "* " << *first << '\n';
-		for (It it = first + 1; it != topo.end(); ++it) {
-			unsigned indeg = in_degree(*it, g);
-			unsigned outdeg = out_degree(*it, g);
-			sum -= indeg;
-
-			if (opt::verbose > 3)
-				cerr << *it << '\t' << indeg << '\t' << outdeg
-					<< '\t' << sum
-					<< '\t' << sum + (int)outdeg << '\n';
-
-			if (indeg == 0 || sum < 0)
-				break;
-			if (sum == 0) {
-				It last = it + 1;
-				if (isBubble(g, first, last)) {
-					if (opt::verbose > 3)
-						cerr << "good\n";
-					bubbles.push_back(Bubble(first, last));
-					first = it - 1;
-				}
-				break;
-			}
-
-			if (outdeg == 0)
-				break;
-			sum += outdeg;
-		}
-	}
-	return bubbles;
-}
-
 /** Add distances to a path. */
 static ContigPath addDistance(const Graph& g, const ContigPath& path)
 {
@@ -570,11 +562,15 @@ int main(int argc, char** argv)
 		istringstream arg(optarg != NULL ? optarg : "");
 		switch (c) {
 			case '?': die = true; break;
+			case 'a': arg >> opt::maxBranches; break;
 			case 'b': arg >> opt::maxLength; break;
 			case 'c': arg >> opt::minCoverage; break;
 			case 'g': arg >> opt::graphPath; break;
 			case 'j': arg >> opt::threads; break;
 			case 'k': arg >> opt::k; break;
+			case 'D': arg >> opt::dialign_debug; break;
+			case 'M': arg >> opt::dialign_score; break;
+			case 'P': arg >> opt::dialign_prob; break;
 			case 'p': arg >> opt::identity; break;
 			case 'v': opt::verbose++; break;
 			case OPT_HELP:
@@ -607,14 +603,37 @@ int main(int argc, char** argv)
 		exit(EXIT_FAILURE);
 	}
 
+	init_parameters();
+	set_parameters_dna();
+	para->DEBUG = opt::dialign_debug;
+	para->SCR_MATRIX_FILE_NAME = (char*)opt::dialign_score.c_str();
+	para->DIAG_PROB_FILE_NAME = (char*)opt::dialign_prob.c_str();
+	initDialign();
+
 	const char* contigsPath(argv[optind++]);
 	string adjPath(argv[optind++]);
 
+	// Read the contig adjacency graph.
+	if (opt::verbose > 0)
+		cerr << "Reading `" << adjPath << "'...\n";
+	ifstream fin(adjPath.c_str());
+	assert_good(fin, adjPath);
+	Graph g;
+	fin >> g;
+	assert(fin.eof());
+	ContigID::lock();
+	if (opt::verbose > 0)
+		printGraphStats(cerr, g);
+
 	// Read the contigs.
 	Contigs& contigs = g_contigs;
 	if (opt::identity > 0) {
+		if (opt::verbose > 0)
+			cerr << "Reading `" << contigsPath << "'...\n";
 		FastaReader in(contigsPath, FastaReader::NO_FOLD_CASE);
 		for (FastaRecord rec; in >> rec;) {
+			if (ContigID::count(rec.id) == 0)
+				continue;
 			ContigID id(rec.id);
 			assert(contigs.size() == id);
 			contigs.push_back(rec.seq);
@@ -622,16 +641,8 @@ int main(int argc, char** argv)
 		assert(in.eof());
 		assert(!contigs.empty());
 		opt::colourSpace = isdigit(contigs.front()[0]);
-		ContigID::lock();
 	}
 
-	// Read the contig adjacency graph.
-	ifstream fin(adjPath.c_str());
-	assert_good(fin, adjPath);
-	Graph g;
-	fin >> g;
-	assert(fin.eof());
-
 	// Remove contigs with insufficient coverage.
 	if (opt::minCoverage > 0)
 		filterGraph(g);
@@ -654,6 +665,8 @@ int main(int argc, char** argv)
 	else
 		copy(g_popped.begin(), g_popped.end(),
 				ostream_iterator<ContigID>(cout, "\n"));
+	free_prob_dist(pdist);
+	free(para);
 
 	if (opt::verbose > 0)
 		cerr << "Bubbles: " << g_count.bubbles/2
@@ -693,6 +706,8 @@ int main(int argc, char** argv)
 		assert_good(fout, opt::graphPath);
 		write_graph(fout, g, PROGRAM, commandLine);
 		assert_good(fout, opt::graphPath);
+		if (opt::verbose > 0)
+			printGraphStats(cerr, g);
 	}
 
 	return 0;
