@@ -1,14 +1,14 @@
 #include "config.h"
-#include "ConstrainedSearch.h"
-#include "ContigGraph.h"
-#include "ContigGraphAlgorithms.h"
 #include "ContigPath.h"
 #include "Estimate.h"
 #include "IOUtil.h"
 #include "Iterator.h"
-#include "GraphIO.h"
-#include "GraphUtil.h"
 #include "Uncompress.h"
+#include "Graph/ConstrainedSearch.h"
+#include "Graph/ContigGraph.h"
+#include "Graph/ContigGraphAlgorithms.h"
+#include "Graph/GraphIO.h"
+#include "Graph/GraphUtil.h"
 #include <algorithm> // for min
 #include <climits> // for UINT_MAX
 #include <cmath>
@@ -61,7 +61,7 @@ namespace opt {
 	static string out;
 
  	/** Output format */
- 	int format = ADJ; // used by Estimate
+ 	int format = DIST; // used by Estimate
 }
 
 static const char shortopts[] = "d:j:k:o:v";
@@ -136,6 +136,8 @@ int main(int argc, char** argv)
 	string estFile(argv[optind++]);
 
 	// Read the contig adjacency graph.
+	if (opt::verbose > 0)
+		cerr << "Reading `" << adjFile << "'..." << endl;
 	ifstream fin(adjFile.c_str());
 	assert_good(fin, adjFile);
 	Graph g;
@@ -213,6 +215,8 @@ static unsigned calculatePathLength(const Graph& g,
 		const ContigNode& origin,
 		const ContigPath& path, size_t prefix = 0, size_t suffix = 0)
 {
+	if (prefix + suffix == path.size())
+		return 0;
 	assert(prefix + suffix < path.size());
 	int length = addProp(g, path.begin() + prefix,
 			path.end() - suffix).length;
@@ -310,7 +314,8 @@ static ContigPath constructAmbiguousPath(const Graph &g,
 
 		// Account for the overlap on the right.
 		int dist = length + getDistance(g,
-				*(longestPath.rbegin() + longestSuffix),
+				longestSuffix == longestPath.size() ? origin
+				: *(longestPath.rbegin() + longestSuffix),
 				*(longestPath.rbegin() + longestSuffix - 1));
 
 		// Add k-1 because it is the convention.
@@ -358,6 +363,18 @@ map<ContigNode, int> makeDistanceMap(const Graph& g,
 	return distances;
 }
 
+/** Print a distance map. */
+static void printDistanceMap(ostream& out, const Graph& g,
+		const ContigNode& u, const ContigPath& path)
+{
+	typedef map<ContigNode, int> DistanceMap;
+	DistanceMap distanceMap = makeDistanceMap(g, u, path);
+	for (DistanceMap::const_iterator it = distanceMap.begin();
+			it != distanceMap.end(); ++it)
+		out << '"' << u << "\" -> \"" << it->first
+			<< "\" [d=" << it->second << "]\n";
+}
+
 /** Find a path for the specified distance estimates.
  * @param out [out] the solution path
  */
@@ -530,6 +547,9 @@ static void handleEstimate(const Graph& g,
 		assert(solutions.size() == 1);
 		assert(bestSol != solutions.end());
 		ContigPath& path = *bestSol;
+		if (opt::verbose > 1)
+			printDistanceMap(vout, g, origin, path);
+
 		extend(g, path.back(), back_inserter(path));
 		out.insert(out.end(), path.begin(), path.end());
 		stats.uniqueEnd++;
@@ -569,7 +589,7 @@ static void* worker(void* pArg)
 
 		ContigPath path;
 		handleEstimate(*arg.graph, er, true, path);
-		path.reverseComplement();
+		reverseComplement(path.begin(), path.end());
 		path.push_back(ContigNode(er.refID, false));
 		handleEstimate(*arg.graph, er, false, path);
 		if (path.size() > 1) {
@@ -588,6 +608,8 @@ static void* worker(void* pArg)
 static void generatePathsThroughEstimates(const Graph& g,
 		const string& estPath)
 {
+	if (opt::verbose > 0)
+		cerr << "Reading `" << estPath << "'..." << endl;
 	ifstream inStream(estPath.c_str());
 	assert_good(inStream, estPath);
 
